from xbee.frame import Frame

class Reader:
	def __init__(self, port):
		self.port = port

	def read_frame(self):
		try:
			port = self.port

			# read input until start of frame and clear start of frame indicator
			while port.read() != b"<":
				pass
			port.read(2)

			# read frame type and convert it to text
			frame_type_value = ord(port.read())
			if frame_type_value < 128:
				frame_type = "Binary"
			elif frame_type_value < 256:
				frame_type = "ASCII"

			# number of fields
			number_of_fields = ord(port.read())

			#skip_bytes(1)
			port.read()

			# read serial ID until next seperator
			# convert serial ID bytes to string
			serialID = []
			while True:
				byte = port.read()
				if byte == b"#": break
				serialID.append(byte.decode("ascii"))
			serialID = "".join(serialID)
				
			# repeat above for waspmote ID
			waspmoteID = []
			while True:
				byte = port.read()
				if byte == b"#": break
				waspmoteID.append(byte.decode("ascii"))
			waspmoteID = "".join(waspmoteID)


			# convert frame sequence bytes into int
			frame_sequence = []
			while True:
				byte = port.read()
				if byte == b"#": break
				frame_sequence.append(byte.decode())
			frame_sequence = "".join(frame_sequence)

			fields = dict()

			for i in range(number_of_fields):
				key = []
				while True:
					byte = port.read()
					if byte == b":": break
					key.append(byte.decode())

				value = []
				while True:
					byte = port.read()
					if byte == b"#": break
					value.append(byte.decode())

				fields.update({"".join(key):"".join(value)})

			return Frame(frame_type, number_of_fields, serialID, waspmoteID, frame_sequence, fields)

		except KeyboardInterrupt:
			raise

		except Exception:
			return -1

	# Utility

	def skip_bytes(self, n):
		# skip n bytes
		for i in range(n):
			self.port.read()


	# DEBUG
	def read_raw_frame(self):
		port = self.port

		# read input until start of frame
		while port.read() != b"=":
			pass
		port.read()

		frame = [b"<", b"="]
		while True:
			byte = port.read()
			if byte == b"<": break
			frame.append(byte)

		print(frame)